package dcguibuilder;

/**
 * Title:        Direct Csound GUI Builder
 * Description:  program to graphically build GUI's for Gabriel Maldonado's Direct Csound GUI Opcodes
 * Copyright:    Copyright (c) 2001
 * Company:      steven yi music
 * @author steven yi
 * @version 1.0
 */

import javax.swing.*;
import javax.swing.event.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;

public class WidgetContainerEditGUI extends JComponent implements WidgetEditGUI {
  DCGUIMain main = null;
  JLabel labelLabel = new JLabel();
  JLabel iwidthLabel = new JLabel();
  JLabel iheightLabel = new JLabel();
  JLabel ixLabel = new JLabel();
  JLabel iyLabel = new JLabel();
  JLabel iborderLabel = new JLabel();
  JTextField labelText = new JTextField();
  JTextField iwidthText = new JTextField();
  JTextField iheightText = new JTextField();
  JTextField ixText = new JTextField();
  JComboBox borderComboBox = new JComboBox();
  JTextField iyText = new JTextField();
  Vector borderOptions = new Vector();
  WidgetContainer widgetContainer;

  public WidgetContainerEditGUI(WidgetContainer widgetContainer) {
    this.widgetContainer = widgetContainer;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {
    this.setPreferredSize(new Dimension(200, 300));
    labelLabel.setText("label");
    labelLabel.setBounds(new Rectangle(4, 4, 41, 17));
    iwidthLabel.setToolTipText("");
    iwidthLabel.setText("iwidth");
    iwidthLabel.setBounds(new Rectangle(4, 26, 41, 17));
    iheightLabel.setText("iheight");
    iheightLabel.setBounds(new Rectangle(4, 47, 41, 17));
    ixLabel.setText("ix");
    ixLabel.setBounds(new Rectangle(4, 69, 41, 17));
    iyLabel.setText("iy");
    iyLabel.setBounds(new Rectangle(4, 91, 41, 17));
    iborderLabel.setText("iborder");
    iborderLabel.setBounds(new Rectangle(4, 113, 41, 17));
    labelText.setBounds(new Rectangle(52, 4, 124, 21));
    iwidthText.setBounds(new Rectangle(52, 26, 124, 21));
    iheightText.setBounds(new Rectangle(52, 47, 124, 21));
    ixText.setBounds(new Rectangle(52, 69, 124, 21));
    borderComboBox.setBounds(new Rectangle(52, 113, 124, 21));
    iyText.setBounds(new Rectangle(52, 91, 124, 21));
    this.add(labelLabel, null);
    this.add(labelText, null);
    this.add(iwidthLabel, null);
    this.add(iwidthText, null);
    this.add(iheightLabel, null);
    this.add(iheightText, null);
    this.add(ixLabel, null);
    this.add(ixText, null);
    this.add(iyLabel, null);
    this.add(iyText, null);
    this.add(iborderLabel, null);
    this.add(borderComboBox, null);

    borderOptions.add("0 - no border");
    borderOptions.add("1 - down box border");
    borderOptions.add("2 - up box border");
    borderOptions.add("3 - engraved border");
    borderOptions.add("4 - embossed border");
    borderOptions.add("5 - black line border");
    borderOptions.add("6 - thin down border");
    borderOptions.add("7 - thin up border");

    borderComboBox.setModel(new DefaultComboBoxModel(borderOptions));

    populateFields();

    labelText.getDocument().addDocumentListener(new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        widgetContainer.label = labelText.getText();
      }
      public void removeUpdate(DocumentEvent e) {
        widgetContainer.label = labelText.getText();
      }
      public void changedUpdate(DocumentEvent e) {
        widgetContainer.label = labelText.getText();
      }
    });

    iwidthText.getDocument().addDocumentListener(new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        widgetContainer.iwidth = checkNumber(iwidthText.getText());
        main.updateWidgets();
      }
      public void removeUpdate(DocumentEvent e) {
        widgetContainer.iwidth = checkNumber(iwidthText.getText());
        main.updateWidgets();
      }
      public void changedUpdate(DocumentEvent e) {
        widgetContainer.iwidth = checkNumber(iwidthText.getText());
        main.updateWidgets();
      }
    });
    iheightText.getDocument().addDocumentListener(new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        widgetContainer.iheight = checkNumber(iheightText.getText());
        main.updateWidgets();
      }
      public void removeUpdate(DocumentEvent e) {
        widgetContainer.iheight = checkNumber(iheightText.getText());
        main.updateWidgets();
      }
      public void changedUpdate(DocumentEvent e) {
        widgetContainer.iheight = checkNumber(iheightText.getText());
        main.updateWidgets();
      }
    });
    ixText.getDocument().addDocumentListener(new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        widgetContainer.ix = checkNumber(ixText.getText());
        main.updateWidgets();
      }
      public void removeUpdate(DocumentEvent e) {
        widgetContainer.ix = checkNumber(ixText.getText());
        main.updateWidgets();
      }
      public void changedUpdate(DocumentEvent e) {
        widgetContainer.ix = checkNumber(ixText.getText());
        main.updateWidgets();
      }
    });

    iyText.getDocument().addDocumentListener(new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        widgetContainer.iy = checkNumber(iyText.getText());
        main.updateWidgets();
      }
      public void removeUpdate(DocumentEvent e) {
        widgetContainer.iy = checkNumber(iyText.getText());
        main.updateWidgets();
      }
      public void changedUpdate(DocumentEvent e) {
        widgetContainer.iy = checkNumber(iyText.getText());
        main.updateWidgets();
      }
    });

    borderComboBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String text = borderComboBox.getSelectedItem().toString();
        text = text.substring(0,1);
        widgetContainer.iborder = checkNumber(text);
      }
    });
  }

  public int checkNumber(String input) {
    int x;
    try {
       x = Integer.parseInt(input);
    } catch (NumberFormatException ne) {
      x = 0;
    }
    if (x < 0) { x = 0; }
    return x;
  }

  public void setMain(DCGUIMain main) {
    this.main = main;
  }

  public void populateFields() {
    labelText.setText(widgetContainer.label);
    iwidthText.setText(Integer.toString(widgetContainer.iwidth));
    iheightText.setText(Integer.toString(widgetContainer.iheight));
    ixText.setText(Integer.toString(widgetContainer.ix));
    iyText.setText(Integer.toString(widgetContainer.iy));
    borderComboBox.setSelectedIndex(widgetContainer.iborder);
  }
}