package dcguibuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.LineBorder;
import java.math.*;
import javax.swing.*;
import java.util.*;

/**
 * Title:        Direct Csound GUI Builder
 * Description:  program to graphically build GUI's for Gabriel Maldonado's Direct Csound GUI Opcodes
 * Copyright:    Copyright (c) 2001
 * Company:      steven yi music
 * @author steven yi
 * @version 1.0
 */

public class WidgetContainerGUI extends JComponent implements MouseListener, MouseMotionListener, ActionListener {
  public static int EDGE = 4;

  DCGUIMain main;

  boolean dragAllow = false;
  boolean resizeX = false;
  boolean resizeY = false;
  Point anchor, point;
  WidgetContainer widgetContainer;
  String name;
  int anchorX, anchorY;
  int bufferX, bufferY;

  JPopupMenu popup = new JPopupMenu();
  JMenuItem menuFLpanel = new JMenuItem();
  JMenu menuValuator = new JMenu();
  JMenu menuOther = new JMenu();
  JMenuItem menuFLSlider = new JMenuItem();
  JMenuItem menuFLknob = new JMenuItem();
  JMenuItem menuFLroller = new JMenuItem();
  JMenuItem menuFLtext = new JMenuItem();
  JMenuItem menuFLjoy = new JMenuItem();
  JMenuItem menuFLcount = new JMenuItem();
  JMenuItem menuFLbutton = new JMenuItem();
  JMenuItem menuFLbutBank = new JMenuItem();
  JMenuItem menuFLbox = new JMenuItem();
  JMenuItem menuFLvalue = new JMenuItem();

  public WidgetContainerGUI(WidgetContainer widgetContainer, DCGUIMain main) {
    this.widgetContainer = widgetContainer;
    this.main = main;
    name = widgetContainer.getClass().getName();
    name = name.substring(name.lastIndexOf(".") + 1);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.addMouseListener(this);
    this.addMouseMotionListener(this);
    this.setSize(50,50);
    menuOther.setText("add Widget Other");
    menuValuator.setText("add Widget Valuator");
    menuFLSlider.setText("FLslider");
    menuFLknob.setText("FLknob");
    menuFLroller.setText("FLroller");
    menuFLtext.setText("FLtext");
    menuFLjoy.setText("FLjoy");
    menuFLcount.setText("FLcount");
    menuFLbutton.setText("FLbutton");
    menuFLbutBank.setText("FLbutBank");
    menuFLbox.setText("FLbox");
    menuFLvalue.setText("FLvalue");

    menuFLSlider.addActionListener(this);
    menuFLknob.addActionListener(this);
    menuFLroller.addActionListener(this);
    menuFLtext.addActionListener(this);
    menuFLjoy.addActionListener(this);
    menuFLcount.addActionListener(this);
    menuFLbutton.addActionListener(this);
    menuFLbutBank.addActionListener(this);
    menuFLbox.addActionListener(this);
    menuFLvalue.addActionListener(this);

    popup.add(menuValuator);
    popup.add(menuOther);
    menuValuator.add(menuFLSlider);
    menuValuator.add(menuFLknob);
    menuValuator.add(menuFLroller);
    menuValuator.add(menuFLtext);
    menuValuator.add(menuFLjoy);
    menuValuator.add(menuFLcount);
    menuOther.add(menuFLbutton);
    menuOther.add(menuFLbutBank);
    menuOther.add(menuFLbox);
    menuOther.add(menuFLvalue);
  }


  public void mousePressed(MouseEvent e) {
    if(SwingUtilities.isRightMouseButton(e)) {
      bufferX = e.getX();
      bufferY = e.getY();
      popup.show(this, e.getX(), e.getY());
    } else {
      main.populateProperty(((Widget)widgetContainer).getEditGUI());
      anchorX = e.getX();
      anchorY = e.getY();
      dragAllow = true;
    }
  }

  public void mouseDragged(MouseEvent e) {
    int newX, newY;
    if(dragAllow) {
      if(resizeX && resizeY) {
        newX = this.getWidth();
        newY = this.getY();
        if(e.getX() > EDGE * 2) {
          newX = e.getX();
        }
        if(e.getX() > getParent().getWidth()) {
          newX = getParent().getWidth();
        }
        if(e.getY() > EDGE * 2) {
          newY = e.getY();
        }
        if(e.getY() > getParent().getHeight()) {
          newY = getParent().getHeight();
        }
        this.setSize(newX, newY);
        widgetContainer.iwidth = newX;
        widgetContainer.iheight = newY;
      } else if(resizeX) {
        newX = EDGE * 2;
        if(e.getX() > EDGE * 2) {
          newX = e.getX();
        }
        if(e.getX() > getParent().getWidth()) {
          newX = getParent().getWidth();
        }
        this.setSize(newX, this.getHeight());
        widgetContainer.iwidth = newX;
      } else if(resizeY) {
        newY = EDGE * 2;
        if(e.getY() > EDGE * 2) {
          newY = e.getY();
        }
        if(e.getY() > getParent().getHeight()) {
          newY = getParent().getHeight();
        }
        this.setSize(this.getWidth(), newY);
        widgetContainer.iheight = newY;
      } else {
        point = SwingUtilities.convertPoint(this, e.getPoint(), getParent());
        newX = point.x - anchorX;
        newY = point.y - anchorY;

        if(newX < 0 ) {
          newX = 0;
        } else if(newX > getParent().getWidth() - this.getWidth()) {
          newX = getParent().getWidth() - this.getWidth();
        }

        if(newY < 0 ) {
          newY = 0;
        } else if(newY > getParent().getHeight() - this.getHeight()) {
          newY = getParent().getHeight() - this.getHeight();
        }

        this.setLocation(newX, newY);
        widgetContainer.ix = newX;
        widgetContainer.iy = newY;
      }
       revalidate();
       repaint();
    }
  }

  public void mouseMoved(MouseEvent e) {
    Object source = e.getSource();
    int xValue = e.getX();
    int yValue = e.getY();
    int width = this.getWidth();
    int height = this.getHeight();

    if((xValue > width - EDGE) && (yValue > height - EDGE )) {
      setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
      resizeX = true;
      resizeY = true;
    } else if(xValue > width - EDGE) {
      setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
      resizeX = true;
      resizeY = false;
    } else if(yValue > height - EDGE) {
      setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
      resizeX = false;
      resizeY = true;
    } else if(source == this) {
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      resizeX = false;
      resizeY = false;
    }
  }

  public void mouseReleased(MouseEvent e) {
    main.populateProperty(((Widget)widgetContainer).getEditGUI());
    dragAllow = false;

  }

  public void mouseExited(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}

  public void actionPerformed(ActionEvent ae) {
    WidgetValuator tempWidget;
    WidgetOther tempOWidget;
    try {
      tempWidget = (WidgetValuator)(Class.forName("dcguibuilder." + ae.getActionCommand()).newInstance());
      widgetContainer.addWidget((Widget)tempWidget);
      WidgetValuatorGUI temp = new WidgetValuatorGUI(tempWidget, main);
      this.add(temp);
      tempWidget.ix = bufferX;
      tempWidget.iy = bufferY;
      temp.setLocation(bufferX, bufferY);
    } catch (ClassCastException ce) {
      try {
        tempOWidget = (WidgetOther)(Class.forName("dcguibuilder." + ae.getActionCommand()).newInstance());
        widgetContainer.addWidget((Widget)tempOWidget);
        WidgetOtherGUI temp = new WidgetOtherGUI(tempOWidget, main);
        this.add(temp);
        tempOWidget.ix = bufferX;
        tempOWidget.iy = bufferY;
        temp.setLocation(bufferX, bufferY);
      } catch (Exception e) {
        System.out.println("damn");
      }
    } catch (Exception e) {
        System.out.println("damn");
    }
  }

  public void populate() {
    ArrayList tempWidgets = widgetContainer.getWidgets();
    int size = tempWidgets.size();
    for(int i = 0; i < size; i++) {
      Widget tempWidget = (Widget)tempWidgets.get(i);
      if(tempWidget instanceof WidgetValuator) {
        WidgetValuator tempWidgetValuator = (WidgetValuator)tempWidget;
        WidgetValuatorGUI temp = new WidgetValuatorGUI(tempWidgetValuator, main);
        this.add(temp);
        temp.setLocation(tempWidgetValuator.ix, tempWidgetValuator.iy);
      } else if (tempWidget instanceof WidgetOther) {
        WidgetOther tempWidgetOther = (WidgetOther)tempWidget;
        WidgetOtherGUI temp = new WidgetOtherGUI(tempWidgetOther, main);
        this.add(temp);
        temp.setLocation(tempWidgetOther.ix, tempWidgetOther.iy);
      } else {
        System.err.println("error");
      }
    }
  }

  public void paintComponent(Graphics g) {
    this.setLocation(widgetContainer.ix, widgetContainer.iy);
    this.setSize(widgetContainer.iwidth, widgetContainer.iheight);
    g.setColor(Color.pink);
    g.fillRect(0, 0, widgetContainer.iwidth, widgetContainer.iheight);
    g.setColor(Color.red);
    g.drawRect(0, 0, widgetContainer.iwidth - 1, widgetContainer.iheight - 1);
    g.drawString(name, 7, 15);
  }

}