package dcguibuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.LineBorder;
import java.math.*;
import javax.swing.*;

/**
 * Title:        Direct Csound GUI Builder
 * Description:  program to graphically build GUI's for Gabriel Maldonado's Direct Csound GUI Opcodes
 * Copyright:    Copyright (c) 2001
 * Company:      steven yi music
 * @author steven yi
 * @version 1.0
 */

public class WidgetValuatorGUI extends JComponent implements MouseListener, MouseMotionListener {
  public static int EDGE = 4;

  DCGUIMain main;

  boolean dragAllow = false;
  boolean resizeX = false;
  boolean resizeY = false;
  Point anchor, point;
  WidgetValuator widgetValuator;
  String name;
  int anchorX, anchorY;
  int bufferX, bufferY;

  public WidgetValuatorGUI(WidgetValuator widgetValuator, DCGUIMain main) {
    this.widgetValuator = widgetValuator;
    this.main = main;
    name = widgetValuator.getClass().getName();
    name = name.substring(name.lastIndexOf(".") + 1);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.addMouseListener(this);
    this.addMouseMotionListener(this);
    this.setSize(50,50);
  }


  public void mousePressed(MouseEvent e) {
    main.populateProperty(((Widget)widgetValuator).getEditGUI());
    anchorX = e.getX();
    anchorY = e.getY();
    dragAllow = true;
  }

  public void mouseDragged(MouseEvent e) {
    int newX, newY;
    if(dragAllow) {
      if(resizeX && resizeY) {
        newX = this.getWidth();
        newY = this.getY();
        if(e.getX() > EDGE * 2) {
          newX = e.getX();
        }
        if(e.getX() > getParent().getWidth()) {
          newX = getParent().getWidth();
        }
        if(e.getY() > EDGE * 2) {
          newY = e.getY();
        }
        if(e.getY() > getParent().getHeight()) {
          newY = getParent().getHeight();
        }
        this.setSize(newX, newY);
        widgetValuator.iwidth = newX;
        widgetValuator.iheight = newY;
      } else if(resizeX) {
        newX = EDGE * 2;
        if(e.getX() > EDGE * 2) {
          newX = e.getX();
        }
        if(e.getX() > getParent().getWidth()) {
          newX = getParent().getWidth();
        }
        this.setSize(newX, this.getHeight());
        widgetValuator.iwidth = newX;
      } else if(resizeY) {
        newY = EDGE * 2;
        if(e.getY() > EDGE * 2) {
          newY = e.getY();
        }
        if(e.getY() > getParent().getHeight()) {
          newY = getParent().getHeight();
        }
        this.setSize(this.getWidth(), newY);
        widgetValuator.iheight = newY;
      } else {
        point = SwingUtilities.convertPoint(this, e.getPoint(), getParent());
        newX = point.x - anchorX;
        newY = point.y - anchorY;

        if(newX < 0 ) {
          newX = 0;
        } else if(newX > getParent().getWidth() - this.getWidth()) {
          newX = getParent().getWidth() - this.getWidth();
        }

        if(newY < 0 ) {
          newY = 0;
        } else if(newY > getParent().getHeight() - this.getHeight()) {
          newY = getParent().getHeight() - this.getHeight();
        }

        this.setLocation(newX, newY);
        widgetValuator.ix = newX;
        widgetValuator.iy = newY;
      }
       revalidate();
       repaint();
    }
  }

  public void mouseMoved(MouseEvent e) {
    Object source = e.getSource();
    int xValue = e.getX();
    int yValue = e.getY();
    int width = this.getWidth();
    int height = this.getHeight();

    if((xValue > width - EDGE) && (yValue > height - EDGE )) {
      setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
      resizeX = true;
      resizeY = true;
    } else if(xValue > width - EDGE) {
      setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
      resizeX = true;
      resizeY = false;
    } else if(yValue > height - EDGE) {
      setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
      resizeX = false;
      resizeY = true;
    } else if(source == this) {
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      resizeX = false;
      resizeY = false;
    }
  }

  public void mouseReleased(MouseEvent e) {
    main.populateProperty(((Widget)widgetValuator).getEditGUI());
    dragAllow = false;

  }

  public void mouseExited(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}

  public void paintComponent(Graphics g) {
    this.setLocation(widgetValuator.ix, widgetValuator.iy);
    this.setSize(widgetValuator.iwidth, widgetValuator.iheight);
    g.setColor(new Color(230, 235, 235));
    g.fillRect(0, 0, widgetValuator.iwidth, widgetValuator.iheight);
    g.setColor(Color.blue);
    g.drawRect(0, 0, widgetValuator.iwidth - 1, widgetValuator.iheight - 1);
    g.drawString(name, 7, 15);
  }

}

